/*
 * This file is part of the authRXBN single sign-on package.
 *
 * (c) Ruben Meyer <contact@rxbn.de>
 */

var express = require('express');
var route = express.Router();

route.post('/register', (req, res) => {
	if(!global['app'].cfg.web.registration) {
		return res.type('json').status(400).end(JSON.stringify({status: 400, message: "msg.auth.registration.deactivated"}));
	}
	// TODO: register
});

route.post('/login', (req, res) => {
	/*
	 * done - check body vars -> else ERR 401 'msg.auth.login.failed'
	 * done - get users by mail / nickname -> else ERR 401 'msg.auth.login.failed' OR ERR 500 'msg.database.error'
	 * done - validate password hash -> else ERR 401 'msg.auth.login.failed'
	 * TODO - add new activity 'action.user.login'
	 */

	if(req.session.user) {
		return res.type('json').end(JSON.stringify({
			status: 401,
			message: 'msg.auth.logout.required'
		}));
	}

	// check body variables
	if(!req.body.email && !req.body.password) {
		return res.type('json').status(401).end(JSON.stringify({
			status: 401,
			message: [
				'msg.request.data.missing',
				'msg.auth.login.failed'
			]
		}));
	}
	let email = req.body.email;
	let pass = req.body.password;

	global['modules'].database.getUser(email, (err, rep) => {
		console.log(err, rep);
		if(err) {
			global['logs'].debug(err);
			return res.type('json').status(500).end(JSON.stringify({
				status: 500,
				message: [
					'msg.database.error',
					'msg.auth.login.failed'
				]
			}));
		}
		if(!rep || rep === null || rep.length == 0 || rep.length > 1 || !global['modules'].auth.validateHash(rep[0].passhash, pass)) {
			return res.type('json').status(401).end(JSON.stringify({
				status: 401,
				message: 'msg.auth.login.failed'
			}));
		} else {
			// add cookies; login
			// new activity 'action.user.login'

			// add session data
			req.session.user = {
				'id': rep[0]._id,
				'group': rep[0].group
			};

			return res.type('json').end(JSON.stringify({
				status: 200,
				message: 'msg.auth.login.successful',
				type: 'form' // TODO: types - { form, access_app}
			}));
		}
	});
});

route.post('/authenticate', (req, res) => {
	// TODO: authenticate
});

route.get('/logout', (req, res) => {
	if(!req.session.user) {
		return res.type('json').end(JSON.stringify({
			status: 401,
			message: 'msg.auth.login.required'
		}));
	} else {
		res.clearCookie('RememberMe');
		req.session.destroy();
		return res.type('json').end(JSON.stringify({
			status: 200,
			message: 'msg.auth.logout.successful'
		}));
	}
});

if(global['gds'].debug) {
	// DEBUG info
	route.get('/info', (req, res) => {
		let obj = {};
		if(req.session) obj.session = req.session;
		if(req.cookies) obj.cookie = req.cookies;
		res.type('json').end(JSON.stringify(obj));
	});
}

module.exports = route;
